#include "COMMON.H"

struct MD3HEADER
{
	char	ID[4];
	int		nVersion;
	char	Filename[68];
	int		nBoneFrameNum;
	int		nTagNum;
	int		nMeshNum;
	int		nMaxSkinNum;
	int		nHeaderLenght;
	int		nTagStart;
	int		nTagEnd;
	int		nFilesize;
};

struct MD3TAG
{
	char	Name[64];
	float	Position[3];
	float	Rotation[3][3];
};

struct MD3BONEFRAME
{
	float	Max[3];
	float	Min[3];
	float	Position[3];
	float	scale;
	char	Creator[16];
};

struct MD3VERTICE
{
	signed short	nVec[3];
	unsigned char	EnvTex[2];
};

struct MD3TRIANGLE
{
	int nVertices[3];
};

struct MD3TEXTURE
{
	float fTexture[2];
};

struct MD3MESH
{
	char	ID[4];
	char	Name[68];
	int		nFrameNum;
	int		nSkinNum;
	int		nVertexNum;
	int		nTriangleNum;
	int		nTriangleStart;
	int		nHeaderSize;
	int		nTexVecStart;
	int		nVertexStart;
	int		nMeshSize;
	char	Data[0x10000];
	unsigned int nTexture;
};

MD3HEADER		md3_header;
MD3TAG			*md3_tag;
MD3BONEFRAME	md3_boneframe[256];
MD3MESH			*md3_mesh;
int				md3_nTagCount;

void MD3_LoadSkins(int nMesh)
{
	FILE *stream;
	short height;
	short width;
	char type;
	unsigned char *pData;
	char strFile[256];
	char strDrive[16];
	char strDir[256];
	char strExt[10];
	unsigned char R;
	unsigned char G;
	unsigned char B;
	int i;
	memcpy(strFile, md3_mesh[nMesh].Data, 68);
	_splitpath(strFile, strDrive, strDir, strFile, strExt); 
	//This isn't good... change this
	strcpy(strDir, ".\\MODELS\\");
	strcat(strDir, strFile);
	strcat(strDir, strExt);
	//Better support for TGAs maybe ;)
	md3_mesh[nMesh].nTexture = 0;
	stream = fopen(strDir, "rb");
	if(stream == NULL)
	{
		return;
	}
	fseek(stream, 2, SEEK_SET);
	fread(&type, 1, 1, stream);
	if(type != 2)
	{
		//Other than 2 not supported
		return;
	}
	fseek(stream, 0x0C, SEEK_SET);
	fread(&width, 2, 1, stream);
	fread(&height, 2, 1, stream);
	fseek(stream, 0x12, SEEK_SET);
	pData = new unsigned char[width * height * 3];
	fread(pData, width * height * 3, 1, stream);
	fclose(stream);
	for(i = 0; i < height * width * 3; i += 3)
	{
		R = pData[i + 0];
		G = pData[i + 1];
		B = pData[i + 2];
		pData[i + 0] = B;
		pData[i + 1] = G;
		pData[i + 2] = R;
	}
	glGenTextures(1, &md3_mesh[nMesh].nTexture);
	glBindTexture(GL_TEXTURE_2D, md3_mesh[nMesh].nTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, pData);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	delete pData;
}

void MD3_Load(char *lpstrFile)
{
	FILE *stream;
	int i;
	stream = fopen(lpstrFile, "rb");
	fread(&md3_header, sizeof(MD3HEADER), 1, stream);
	fread(&md3_boneframe, sizeof(MD3BONEFRAME) * md3_header.nBoneFrameNum, 1, stream);
	md3_nTagCount = md3_header.nTagNum * md3_header.nBoneFrameNum;
	fread(&md3_tag, sizeof(MD3TAG) * md3_nTagCount, 1, stream);
	for(i = 0; i < md3_header.nMeshNum; i++)
	{
		fread(&md3_mesh[i], 0x6C, 1, stream);
		fread(&md3_mesh[i].Data, md3_mesh[i].nMeshSize - 0x6C, 1, stream);
		if(md3_mesh[i].nSkinNum != 0)
		{
			MD3_LoadSkins(i);
		}
	}
	fclose(stream);
}

GLfloat LightAmbient[]=		{ 0.5f, 0.5f, 0.5f, 1.0f };
GLfloat LightDiffuse[]=		{ 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat LightPosition[]=	{ 0.0f, 0.0f, 7.0f, 1.0f };

void MD3_CreateDisplayList(int *DL, float fScaleFactor)
{
	int i, j;
	int nVert1, nVert2, nVert3;
	MD3TRIANGLE *pTris;
	MD3VERTICE	*pVert;
	MD3TEXTURE	*pText;
	*DL = glGenLists(1);
	glNewList(*DL, GL_COMPILE);
	glPushMatrix();
	glScalef(fScaleFactor, fScaleFactor, fScaleFactor);
	glBegin(GL_TRIANGLES);
	{
		for(j = 0; j < md3_header.nMeshNum; j++)
		{
			pTris = (MD3TRIANGLE*)((unsigned char*)&md3_mesh[j] + md3_mesh[j].nTriangleStart);
			pVert = (MD3VERTICE*)((unsigned char*)&md3_mesh[j] + md3_mesh[j].nVertexStart);
			pText = (MD3TEXTURE*)((unsigned char*)&md3_mesh[j] + md3_mesh[j].nTexVecStart);
			for(i = 0; i < md3_mesh[j].nTriangleNum; i++)
			{
				nVert1 = pTris[i].nVertices[0];
				nVert2 = pTris[i].nVertices[1];
				nVert3 = pTris[i].nVertices[2];

				glBindTexture(GL_TEXTURE_2D, md3_mesh[j].nTexture);
				//Normal calculation
				float px, py, pz;
				float qx, qy, qz;
				float nx, ny, nz;
				
				px = (float)pVert[pTris[i].nVertices[2]].nVec[0] - (float)pVert[pTris[i].nVertices[1]].nVec[0];
				py = (float)pVert[pTris[i].nVertices[2]].nVec[1] - (float)pVert[pTris[i].nVertices[1]].nVec[1];
				pz = (float)pVert[pTris[i].nVertices[2]].nVec[2] - (float)pVert[pTris[i].nVertices[1]].nVec[2];

				qx = (float)pVert[pTris[i].nVertices[0]].nVec[0] - (float)pVert[pTris[i].nVertices[1]].nVec[0];
				qy = (float)pVert[pTris[i].nVertices[0]].nVec[1] - (float)pVert[pTris[i].nVertices[1]].nVec[1];
				qz = (float)pVert[pTris[i].nVertices[0]].nVec[2] - (float)pVert[pTris[i].nVertices[1]].nVec[2];

				nx = (py * qz) - (pz * qz);
				ny = (pz * qx) - (pz * qx);
				nz = (px * qy) - (px * qy);

				//nx /= abs(nx) * -1.0f;
				//ny /= abs(ny) * -1.0f;
				//nz /= abs(nz) * -1.0f;

				//glNormal3f(nx, ny, nz);

				glTexCoord2d(pText[nVert1].fTexture[0], 1.0f - pText[nVert1].fTexture[1]);
				glVertex3i(pVert[pTris[i].nVertices[0]].nVec[0], pVert[pTris[i].nVertices[0]].nVec[1], pVert[pTris[i].nVertices[0]].nVec[2]);

				glTexCoord2d(pText[nVert2].fTexture[0], 1.0f - pText[nVert2].fTexture[1]);
				glVertex3i(pVert[pTris[i].nVertices[1]].nVec[0], pVert[pTris[i].nVertices[1]].nVec[1], pVert[pTris[i].nVertices[1]].nVec[2]);

				glTexCoord2d(pText[nVert3].fTexture[0], 1.0f - pText[nVert3].fTexture[1]);
				glVertex3i(pVert[pTris[i].nVertices[2]].nVec[0], pVert[pTris[i].nVertices[2]].nVec[1], pVert[pTris[i].nVertices[2]].nVec[2]);
			}
		}
	}
	glEnd();
	glPopMatrix();
	glEndList();
}

void MD3_CreateDisplayListForMesh(char *strFile, int *DL, float fScaleFactor)
{
	md3_mesh = new MD3MESH[256];
	md3_tag = new MD3TAG[1024];
	memset(&md3_header, 0, sizeof(MD3HEADER));
	memset(md3_tag, 0, sizeof(MD3TAG) * 1024);
	memset(md3_boneframe, 0, sizeof(MD3BONEFRAME) * 256);
	memset(md3_mesh, 0, sizeof(MD3MESH) * 256);
	md3_nTagCount = 0;
	MD3_Load(strFile);
	MD3_CreateDisplayList(DL, fScaleFactor);
	delete md3_mesh;
	delete md3_tag;
}