#include "COMMON.H"

unsigned char	CONSOLE_lpBuffer[0x10000];
int				CONSOLE_nDestroyTextConsole = 0;
int				CONSOLE_nDisableButtons = 0;
HANDLE			CONSOLE_hTextDlg = NULL;
DWORD			CONSOLE_dwTextDlgID = NULL;
HWND			CONSOLE_hTextDlgWnd = NULL;
int				CONSOLE_nPosition = 0;
int				CONSOLE_nGraphicFont = 0;
int				CONSOLE_nGraphicBackground = 0;
int				(*CONSOLE_ConsoleCallback)(char*, void*) = NULL;
void*			CONSOLE_pCallbackParam = NULL;

extern "C" int	XBUILD_GFX_GetScreenWidth();
extern "C" int	XBUILD_GFX_GetScreenHeight();

void CONSOLE_UpdateTextConsole()
{
	SetDlgItemText(CONSOLE_hTextDlgWnd, IDC_CONSOLE, (char*)CONSOLE_lpBuffer);
}

extern "C" void DLLEXPORT XBUILD_CONSOLE_PrintFormatted(const char *lpstrFormat, ...)
{
	char string[256];
	va_list args;
	memset(string, 0, 256);
	va_start(args, lpstrFormat);
	vsprintf(string, lpstrFormat, args);
	va_end(args);
	memmove(CONSOLE_lpBuffer + CONSOLE_nPosition, string, strlen(string));
	CONSOLE_nPosition += strlen(string);
	if(CONSOLE_hTextDlg != NULL)
	{
		CONSOLE_UpdateTextConsole();
	}
}

void CONSOLE_ProcessCommand(char *lpstrCommand)
{
	if(CONSOLE_ConsoleCallback != NULL)
	{
		if(CONSOLE_ConsoleCallback(lpstrCommand, CONSOLE_pCallbackParam))
		{
			return;
		}
	}
	XBUILD_CONSOLE_PrintFormatted("Unknown command: %s\r\n", lpstrCommand);
}

int WINAPI CONSOLE_TextDlgProc(HWND hWnd, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uiMsg)
	{
	case WM_INITDIALOG:
		CONSOLE_hTextDlgWnd = hWnd;
		if(CONSOLE_nDisableButtons)
		{
			EnableWindow(GetDlgItem(hWnd, IDC_INPUT), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_SEND), FALSE);
		}
		CONSOLE_UpdateTextConsole();
		break;
	case WM_COMMAND:
		switch(wParam)
		{
		case 2:
			CONSOLE_nDestroyTextConsole = 1;
			DestroyWindow(hWnd);
			break;
		case IDC_SEND:
			char lpstrCommand[256];
			GetDlgItemText(hWnd, IDC_INPUT, lpstrCommand, 256);
			CONSOLE_ProcessCommand(lpstrCommand);
			SetDlgItemText(hWnd, IDC_INPUT, "");
			break;
		}
		break;
	}
	return FALSE;
}

unsigned long WINAPI CONSOLE_ThrConsoleDlg(void *)
{
	MSG msg;
	CreateDialog((HMODULE)XBUILD_hMod, MAKEINTRESOURCE(IDD_CONSOLE), NULL, (DLGPROC)CONSOLE_TextDlgProc);
	while(!CONSOLE_nDestroyTextConsole)
	{
		if(!GetMessage(&msg, CONSOLE_hTextDlgWnd, 0, 0))
		{
			DestroyWindow(CONSOLE_hTextDlgWnd);
			break;
		}
		if(!IsDialogMessage(CONSOLE_hTextDlgWnd, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	return 0xDEADBEEF;
}

extern "C" void DLLEXPORT XBUILD_CONSOLE_Clear()
{
	memset(CONSOLE_lpBuffer, 0, 0x10000);
}

extern "C" void DLLEXPORT XBUILD_CONSOLE_SetConsoleCallback(void *pCallback, void* pParam)
{
	CONSOLE_ConsoleCallback = (int(*)(char*, void*))pCallback;
	CONSOLE_pCallbackParam = pParam;
}

extern "C" void DLLEXPORT XBUILD_CONSOLE_ShowTextConsole(int nDisabled)
{
	CONSOLE_nDestroyTextConsole = 0;
	CONSOLE_hTextDlg = CreateThread(NULL, NULL, CONSOLE_ThrConsoleDlg, NULL, NULL, &CONSOLE_dwTextDlgID);
	CONSOLE_nDisableButtons = nDisabled;
}

extern "C" void DLLEXPORT XBUILD_CONSOLE_HideTextConsole()
{
	CONSOLE_nDestroyTextConsole = 1;
	TerminateThread(CONSOLE_hTextDlg, 0xDEADBEEF);
	//DestroyWindow(CONSOLE_hTextDlgWnd);
	//WaitForSingleObject(CONSOLE_hTextDlg, INFINITE);
}

extern "C" void DLLEXPORT XBUILD_CONSOLE_SetGraphicConsoleParams(int nFontbase, int nBackground)
{
	CONSOLE_nGraphicFont = nFontbase;
	CONSOLE_nGraphicBackground = nBackground;
}

extern "C" void DLLEXPORT XBUILD_CONSOLE_GetGraphicConsole(XBUILDDISPLAYLIST *dl)
{
	int i, j;
	int nX, nY;
	unsigned char bValue;
	XBUILD_ART_TILEDATA Tile;
	XBUILDDISPLAYELEMENT_TILE DTile;
	nX = 0;
	nY = 0;
	memset(dl, 0, sizeof(XBUILDDISPLAYLIST));
	XBUILD_ART_GetTileData(CONSOLE_nGraphicBackground, &Tile);
	if(XBUILD_GFX_GetScreenWidth() < 640 || XBUILD_GFX_GetScreenHeight() < 480)
	{
		dl->dwReferentialX = XBUILD_GFX_GetScreenWidth();
		dl->dwReferentialY = XBUILD_GFX_GetScreenHeight();
	}
	else
	{
		dl->dwReferentialX = XBUILD_GFX_GetScreenWidth() / 2;
		dl->dwReferentialY = XBUILD_GFX_GetScreenHeight() / 2;
	}
	for(j = 0; j < (int)((dl->dwReferentialY / 2) / Tile.Height); j++)
	{
		for(i = 0; i < (int)(dl->dwReferentialX / Tile.Width) + 1; i++)
		{
			DTile.dwX = i * Tile.Width;
			DTile.dwY = nY;
			DTile.dwTile = CONSOLE_nGraphicBackground;
			memcpy(dl->Element[dl->nElementCount].pData, &DTile, sizeof(XBUILDDISPLAYELEMENT_TILE));
			dl->Element[dl->nElementCount].dwType = XBUILDDISPLAYLIST_TYPE_TILE;
			dl->nElementCount++;
		}
		nY += Tile.Height;
	}
	nY = 0;
	for(i = 0; i < CONSOLE_nPosition; i++)
	{
		bValue = CONSOLE_lpBuffer[i];
		if(bValue == 0x20)
		{
			nX += 5;
		}
		else if(bValue == 0x0A)
		{
			nY += 8;
			nX = 0;
		}
		else if(bValue == 0x0D)
		{
			//Ignore this
		}
		else
		{
			XBUILD_ART_GetTileData(CONSOLE_nGraphicFont + bValue, &Tile);
			DTile.dwTile = CONSOLE_nGraphicFont + bValue;
			DTile.dwX = nX;
			DTile.dwY = nY;
			memcpy(&dl->Element[dl->nElementCount].pData, &DTile, sizeof(XBUILDDISPLAYELEMENT_TILE));
			dl->Element[dl->nElementCount].dwType = XBUILDDISPLAYLIST_TYPE_TILE;
			dl->nElementCount++;
			nX += Tile.Width;
		}
	}
}